/*****************************************************************
 *  File: weather_interface.c
 *
 *  Summary: Functions to draw the weather_interface
 *
 ****************************************************************/
#include <stdio.h>
#include "driver/d_Display_DMA2D.h"
#include "application/weather_interface.h"


/*****************************************************************
 *
 * Function: void draw_weather_interface(char* ip_addr)
 *
 * Summary: Draws the background for the weather station
 *
 * Parameters:
 *
 * Return:
 *
 * Revision History:
 * Creation Date: 28062021
 *
 *****************************************************************/
void draw_weather_interface(char* ip_addr){
	DMA2D_Fill_Color(0xFFFF, Layer_1, Buffer_1);
	DMA2D_Draw_Image((HDP-Display21_Logo_Y_43Inch.width)/2, (VDP-Display21_Logo_Y_43Inch.height)/2, Display21_Logo_Y_43Inch.width, Display21_Logo_Y_43Inch.height,
			0x5F, REPLACE_ALPHA_VALUE, (uint32_t)Display21_Logo_Y_43Inch.data, CM_RGB565, Layer_1, Buffer_1);
	//temperature box
	DMA2D_Draw_Rectangle(0, 0, HDP*2/5, VDP/2 - 10, 0xCF07, 6, Layer_1, Buffer_1);
	DMA2D_write_string("Temperature:", 10, 15, 0x95D6, &courier_new_15_2, Layer_1, Buffer_1, Layer_1, Buffer_1);
	//humidity box
	DMA2D_Draw_Rectangle(0, VDP/2-3-10, HDP*2/5, VDP/2 -27+10, 0xCF07, 6, Layer_1, Buffer_1);
	DMA2D_write_string("Humidity:", 10, VDP/2 - 10 +15, 0x95D6, &courier_new_15_2, Layer_1, Buffer_1, Layer_1, Buffer_1);
	//pressure box
	DMA2D_Draw_Rectangle(HDP*2/5-3, 0, HDP*3/5+3, VDP*2/3 - 20, 0xCF07, 6, Layer_1, Buffer_1);
	DMA2D_write_string("Pressure:", HDP*2/5 +10, 15, 0x95D6, &courier_new_15_2, Layer_1, Buffer_1, Layer_1, Buffer_1);
	DMA2D_write_string("approx. Altitude:", HDP*2/5 +30, VDP/2-20, 0x8000, &courier_new_14, Layer_1, Buffer_1, Layer_1, Buffer_1);
	//gas resistance box
	DMA2D_Draw_Rectangle(HDP*2/5-3, VDP*2/3-3-20, HDP*3/5+3, VDP*1/3 -27+20, 0xCF07, 6, Layer_1, Buffer_1);
	DMA2D_write_string("Gas Resistance:", HDP*2/5 +10, VDP*2/3 +15-20, 0x95D6, &courier_new_15_2, Layer_1, Buffer_1, Layer_1, Buffer_1);

	//IP-Address
	DMA2D_write_string("IP:", 10, VDP-28, 0xBDEF, &courier_new_15_2, Layer_1, Buffer_1, Layer_1, Buffer_1);
	DMA2D_write_string(ip_addr, 50, VDP-28, 0xBDEF, &courier_new_15_2, Layer_1, Buffer_1, Layer_1, Buffer_1);
}

/*****************************************************************
 *
 * Function: draw_weather_data(weather_data_struct w_data)
 *
 * Summary: Draws the values retrieved from the ESP32 on Layer 2 of the display
 *
 * Parameters: -
 *
 *
 * Return: -
 *
 * Revision History:
 * Creation Date: 28062021
 *
 *****************************************************************/
void draw_weather_data(weather_data_struct w_data){
	char temp[11];
	DMA2D_Fill_Color(0x7FFF, Layer_2, Buffer_1);
	//temperature
	sprintf(temp, "%4.2f %cC", w_data.temperature, 0x7f);
	//DMA2D_Draw_FilledRectangle(20, VDP*1/8, HDP*2/5-50, VDP*2/8, 0xFFFFFFFF, Layer_2, Buffer_1);
	DMA2D_write_string(temp, HDP/5 -25*4, VDP/5, 0xFF000000, &arial_28_deg, Layer_2, Buffer_1, Layer_2, Buffer_1);

	//humidity
	sprintf(temp, "%2.0f %%", w_data.humidity);
	//DMA2D_Draw_FilledRectangle(20, VDP*5/8, HDP*2/5-50, VDP*2/8, 0xFFFFFFFF, Layer_2, Buffer_1);
	DMA2D_write_string(temp, HDP/5 -arial_28_deg.chars->image->width*2, VDP*2/3, 0xFF000000, &arial_28_deg, Layer_2, Buffer_1, Layer_2, Buffer_1);

	//pressure
	sprintf(temp, "%6.2f hPa", w_data.pressure);
	//DMA2D_Draw_FilledRectangle(HDP*2/5+20, VDP/5, HDP*3/5-50, VDP*1/6, 0xFFFFFFFF, Layer_2, Buffer_1);
	DMA2D_write_string(temp, HDP*7/10 -arial_28_deg.chars->image->width*6, VDP/5, 0xFF000000, &arial_28_deg, Layer_2, Buffer_1, Layer_2, Buffer_1);

	//approx altitude
	sprintf(temp, "%3.0f m", w_data.approx_altitude);
	//DMA2D_Draw_FilledRectangle(HDP*2/5+20+13*20, VDP/2-10, HDP/4-10, VDP/6-5, 0xFFFFFFFF, Layer_2, Buffer_1);
	DMA2D_write_string(temp, HDP*2/5 +30+13*20, VDP/2-20, 0xFF000000, &courier_new_15_2, Layer_2, Buffer_1, Layer_2, Buffer_1);

	//gas resistance
	sprintf(temp, "%5.2f kOhm", w_data.gas_resistance);
	//DMA2D_Draw_FilledRectangle(HDP*2/5+20, VDP*9/12, HDP*2/5, VDP*1/6, 0xFFFFFFFF, Layer_2, Buffer_1);
	DMA2D_write_string(temp, HDP*7/10 -courier_new_15_2.chars->image->width*6, VDP*3/4+30-25, 0xFF000000, &courier_new_15_2,
			Layer_2, Buffer_1, Layer_2, Buffer_1);
}











