/***********************************************************
 *
 * d_SDRAM_FLASH.c
 * SYSTART GmbH
 * SAM, 23.06.2017
 * RAT, 2018 (NOR_Flash functions)
 *
 ***********************************************************/

/***********************************************************
 * Includes
 ***********************************************************/
#include "driver\d_SDRAM_FLASH.h"
#include "stm32f4xx_fmc.h"

/***********************************************************
 * Declaration internal functions
 ***********************************************************/
void SDRAM_NOR_FLASH_GPIOs_init ( void );
void SDRAM_Initialization_Sequence ( void );
//void NOR_Initialization_Sequence ( void );


/***********************************************************
 * Declaration internal macros
 ***********************************************************/
#define ADDR_SHIFT(A) (NOR_FLASH_START_ADDR + (2 * (A)))
//#define NOR_WRITE(Address, Data)  (*(__IO uint16_t *)(Address) = (Data))

/***********************************************************
 * Function definitions
 ***********************************************************/
void SDRAM_FLASH_init ( void )
{
	FMC_SDRAMTimingInitTypeDef FMC_SDRAMTimingStruct;
	FMC_SDRAMInitTypeDef FMC_SDRAMInitStruct;

	//FMC_NORSRAMTimingInitTypeDef FMC_NORTimingStruct;
	//FMC_NORSRAMInitTypeDef FMC_NORInitStruct;

	// Initialize GPIOs for SDRAM
	SDRAM_NOR_FLASH_GPIOs_init();

	// Activate clock
	RCC_AHB3PeriphClockCmd( RCC_AHB3Periph_FMC, ENABLE );

	// Initialize SDRAM
	FMC_SDRAMTimingStruct.FMC_LoadToActiveDelay = 2;
	FMC_SDRAMTimingStruct.FMC_ExitSelfRefreshDelay = 6;
	FMC_SDRAMTimingStruct.FMC_SelfRefreshTime = 4;
	FMC_SDRAMTimingStruct.FMC_RowCycleDelay = 6;
	FMC_SDRAMTimingStruct.FMC_WriteRecoveryTime = 2;
	FMC_SDRAMTimingStruct.FMC_RPDelay = 2;
	FMC_SDRAMTimingStruct.FMC_RCDDelay = 2;

	FMC_SDRAMInitStruct.FMC_Bank = FMC_Bank1_SDRAM;
	FMC_SDRAMInitStruct.FMC_ColumnBitsNumber = FMC_ColumnBits_Number_8b;
	FMC_SDRAMInitStruct.FMC_RowBitsNumber = FMC_RowBits_Number_12b;
	FMC_SDRAMInitStruct.FMC_SDMemoryDataWidth = FMC_SDMemory_Width_32b;
	FMC_SDRAMInitStruct.FMC_InternalBankNumber = FMC_InternalBank_Number_4;
	FMC_SDRAMInitStruct.FMC_CASLatency = FMC_CAS_Latency_3;
	FMC_SDRAMInitStruct.FMC_WriteProtection = FMC_Write_Protection_Disable;
	FMC_SDRAMInitStruct.FMC_SDClockPeriod = FMC_SDClock_Period_2;
	FMC_SDRAMInitStruct.FMC_ReadBurst = FMC_Read_Burst_Enable;
	FMC_SDRAMInitStruct.FMC_ReadPipeDelay = FMC_ReadPipe_Delay_0;
	FMC_SDRAMInitStruct.FMC_SDRAMTimingStruct = &FMC_SDRAMTimingStruct;

	FMC_SDRAMInit( &FMC_SDRAMInitStruct );

	SDRAM_Initialization_Sequence();

	// Initialize NOR Flash
	/*FMC_NORTimingStruct.FMC_AddressSetupTime = 2;
	FMC_NORTimingStruct.FMC_AddressHoldTime = 0;
	FMC_NORTimingStruct.FMC_DataSetupTime = 10;
	FMC_NORTimingStruct.FMC_BusTurnAroundDuration = 0;
	FMC_NORTimingStruct.FMC_CLKDivision = 0;
	FMC_NORTimingStruct.FMC_DataLatency = 0;
	FMC_NORTimingStruct.FMC_AccessMode = FMC_AccessMode_B;
		//FMC_NORTimingStruct.FMC_AddressSetupTime = 4;
		//FMC_NORTimingStruct.FMC_AddressHoldTime = 3;
		//FMC_NORTimingStruct.FMC_DataSetupTime = 7;
		//FMC_NORTimingStruct.FMC_BusTurnAroundDuration = 1;
		//FMC_NORTimingStruct.FMC_CLKDivision = 1;
		//FMC_NORTimingStruct.FMC_DataLatency = 0;
		//FMC_NORTimingStruct.FMC_AccessMode = FMC_AccessMode_A;

	FMC_NORInitStruct.FMC_Bank = FMC_Bank1_NORSRAM1;
	FMC_NORInitStruct.FMC_DataAddressMux = FMC_DataAddressMux_Disable;
	FMC_NORInitStruct.FMC_MemoryType = FMC_MemoryType_NOR;
	FMC_NORInitStruct.FMC_MemoryDataWidth = FMC_NORSRAM_MemoryDataWidth_16b;
	FMC_NORInitStruct.FMC_BurstAccessMode = FMC_BurstAccessMode_Disable;
	FMC_NORInitStruct.FMC_WaitSignalPolarity = FMC_WaitSignalPolarity_Low;
	FMC_NORInitStruct.FMC_WrapMode = FMC_WrapMode_Disable;
	FMC_NORInitStruct.FMC_WaitSignalActive = FMC_WaitSignalActive_BeforeWaitState;
	FMC_NORInitStruct.FMC_WriteOperation = FMC_WriteOperation_Enable;
	FMC_NORInitStruct.FMC_WaitSignal = FMC_WaitSignal_Disable;
	FMC_NORInitStruct.FMC_ExtendedMode = FMC_ExtendedMode_Disable;
	FMC_NORInitStruct.FMC_AsynchronousWait = FMC_AsynchronousWait_Disable;
	FMC_NORInitStruct.FMC_WriteBurst = FMC_WriteBurst_Disable;
	FMC_NORInitStruct.FMC_ContinousClock = FMC_CClock_SyncOnly;

	FMC_NORInitStruct.FMC_ReadWriteTimingStruct = &FMC_NORTimingStruct;
	FMC_NORInitStruct.FMC_WriteTimingStruct = &FMC_NORTimingStruct;

	FMC_NORSRAMDeInit(FMC_Bank1_NORSRAM1);
	FMC_NORSRAMInit(&FMC_NORInitStruct);

	NOR_Initialization_Sequence();
	//FMC_NORSRAMDeInit(FMC_Bank1_NORSRAM1);
	NOR_ReturnToReadMode();*/

}

void SDRAM_Write_8b ( uint32_t adr, uint8_t data )
{
	*(uint8_t*)( SDRAM_START_ADDR + adr ) = data;
}

void SDRAM_Write_16b ( uint32_t adr, uint16_t data )
{
	*(uint16_t*)( SDRAM_START_ADDR + adr ) = data;
}

void SDRAM_Write_32b ( uint32_t adr, uint32_t data )
{
	*(uint32_t*)( SDRAM_START_ADDR + adr ) = data;
}

uint8_t SDRAM_Read_8b ( uint32_t adr )
{
	uint8_t ret = 0;

	ret = *(__IO uint8_t*)( SDRAM_START_ADDR + adr );

	return ret;
}

uint16_t SDRAM_Read_16b ( uint32_t adr )
{
	uint16_t ret = 0;

	ret = *(__IO uint16_t*)( SDRAM_START_ADDR + adr );

	return ret;
}

uint32_t SDRAM_Read_32b ( uint32_t adr )
{
	uint32_t ret = 0;

	ret = *(__IO uint32_t*)( SDRAM_START_ADDR + adr );

	return ret;
}

/*void NOR_ReadID ( NOR_IDTypeDef* NOR_ID )
{
	NOR_WRITE(ADDR_SHIFT(0x0555), 0x00AA);
	NOR_WRITE(ADDR_SHIFT(0x02AA), 0x0055);
	NOR_WRITE(ADDR_SHIFT(0x0555), 0x0090);

	NOR_ID->Manufacturer_Code = *(__IO uint16_t *) ADDR_SHIFT(0x0000);
	NOR_ID->Device_Code1 = *(__IO uint16_t *) ADDR_SHIFT(0x0001);
	NOR_ID->Device_Code2 = *(__IO uint16_t *) ADDR_SHIFT(0x000E);
	NOR_ID->Device_Code3 = *(__IO uint16_t *) ADDR_SHIFT(0x000F);
}

NOR_Status NOR_EraseBlock ( uint32_t BlockAddr )
{
	NOR_WRITE(ADDR_SHIFT(0x0555), 0x00AA);
	NOR_WRITE(ADDR_SHIFT(0x02AA), 0x0055);
	NOR_WRITE(ADDR_SHIFT(0x0555), 0x0080);
	NOR_WRITE(ADDR_SHIFT(0x0555), 0x00AA);
	NOR_WRITE(ADDR_SHIFT(0x02AA), 0x0055);
	NOR_WRITE((NOR_FLASH_START_ADDR + BlockAddr), 0x30);

	return ( NOR_GetStatus( BlockErase_Timeout ) );
}

NOR_Status NOR_EraseChip ( void )
{
	NOR_WRITE(ADDR_SHIFT(0x0555), 0x00AA);
	NOR_WRITE(ADDR_SHIFT(0x02AA), 0x0055);
	NOR_WRITE(ADDR_SHIFT(0x0555), 0x0080);
	NOR_WRITE(ADDR_SHIFT(0x0555), 0x00AA);
	NOR_WRITE(ADDR_SHIFT(0x02AA), 0x0055);
	NOR_WRITE(ADDR_SHIFT(0x0555), 0x0010);

	return (NOR_GetStatus(ChipErase_Timeout));
}

NOR_Status NOR_ProgramBuffer ( uint16_t* pBuffer, uint32_t WriteAddr, uint32_t NumHalfwordToWrite )
{
	// Todo: Startadresse hinzufuegen
	uint32_t lastloadedaddress = 0x00;
	uint32_t currentaddress = 0x00;
	uint32_t endaddress = 0x00;

	//!< Initialize variables
	currentaddress = WriteAddr;
	endaddress = WriteAddr + NumHalfwordToWrite - 1;
	lastloadedaddress = WriteAddr;

	//!< Issue unlock command sequence
	NOR_WRITE(ADDR_SHIFT(0x00555), 0x00AA); // 1.unlock command

	NOR_WRITE(ADDR_SHIFT(0x02AA), 0x0055); // 2.unlock command

	//!< Write Write Buffer Load Command
	NOR_WRITE(ADDR_SHIFT(WriteAddr), 0x0025); // 3.sets up the Write to Buffer Program command
	NOR_WRITE(ADDR_SHIFT(WriteAddr), (NumHalfwordToWrite - 1)); // 4. sets up the number of words/bytes to be programmed

	//!< Load Data into NOR Buffer
	while(currentaddress <= endaddress)
	{
		//!< Store last loaded address & data value (for polling)
		lastloadedaddress = currentaddress;

		NOR_WRITE(ADDR_SHIFT(currentaddress), *pBuffer++); // 5. loads the first address and data to be programmed
		currentaddress += 1;
	}

	NOR_WRITE(ADDR_SHIFT(lastloadedaddress), 0x29);

	return(NOR_GetStatus(Program_Timeout));
}

NOR_Status NOR_Write_8b ( uint32_t WriteAddr, uint8_t Data )
{
	NOR_WRITE(ADDR_SHIFT(0x0555), 0x00AA);
	NOR_WRITE(ADDR_SHIFT(0x02AA), 0x0055);
	NOR_WRITE(ADDR_SHIFT(0x0555), 0x00A0);
	NOR_WRITE((NOR_FLASH_START_ADDR + WriteAddr), Data);

	return (NOR_GetStatus(Program_Timeout));
}

NOR_Status NOR_Write_16b ( uint32_t WriteAddr, uint16_t Data )
{
	NOR_WRITE(ADDR_SHIFT(0x0555), 0x00AA);
	NOR_WRITE(ADDR_SHIFT(0x02AA), 0x0055);
	NOR_WRITE(ADDR_SHIFT(0x0555), 0x00A0);
	NOR_WRITE((NOR_FLASH_START_ADDR + WriteAddr), Data);

	return (NOR_GetStatus(Program_Timeout));
}

NOR_Status NOR_Write_32b ( uint32_t WriteAddr, uint32_t Data )
{
	NOR_Status status = NOR_ONGOING;

	NOR_WRITE(ADDR_SHIFT(0x0555), 0x00AA);
	NOR_WRITE(ADDR_SHIFT(0x02AA), 0x0055);
	NOR_WRITE(ADDR_SHIFT(0x0555), 0x00A0);
	NOR_WRITE((NOR_FLASH_START_ADDR + WriteAddr), Data);
	status = NOR_GetStatus(Program_Timeout);

	WriteAddr = WriteAddr + 2;

	NOR_WRITE(ADDR_SHIFT(0x0555), 0x00AA);
	NOR_WRITE(ADDR_SHIFT(0x02AA), 0x0055);
	NOR_WRITE(ADDR_SHIFT(0x0555), 0x00A0);
	NOR_WRITE((NOR_FLASH_START_ADDR + WriteAddr), Data>>16);
	status |= NOR_GetStatus(Program_Timeout);

	return status;
}

NOR_Status NOR_WriteBuffer_8b ( uint8_t* pBuffer, uint32_t WriteAddr, uint32_t NumToWrite )
{
	NOR_Status status = NOR_ONGOING;

	do{
		//!< Transfer data to the memory
		status = NOR_Write_16b(WriteAddr, *pBuffer++);
		WriteAddr = WriteAddr + 2;
		NumToWrite--;
	}while((status == NOR_SUCCESS) && (NumToWrite != 0));

	return (status);
}

NOR_Status NOR_WriteBuffer_16b ( uint16_t* pBuffer, uint32_t WriteAddr, uint32_t NumToWrite )
{
	NOR_Status status = NOR_ONGOING;

	do{
		//!< Transfer data to the memory
		status = NOR_Write_16b(WriteAddr, *pBuffer++);
		WriteAddr = WriteAddr + 2;
		NumToWrite--;
	}while((status == NOR_SUCCESS) && (NumToWrite != 0));

	return (status);
}

NOR_Status NOR_WriteBuffer_32b ( uint32_t* pBuffer, uint32_t WriteAddr, uint32_t NumToWrite )
{
	NOR_Status status = NOR_ONGOING;

	do{
		//!< Transfer data to the memory
		status = NOR_Write_16b(WriteAddr, *pBuffer);
		WriteAddr = WriteAddr + 2;//<--
		status |= NOR_Write_16b(WriteAddr, *pBuffer>>16);
		WriteAddr = WriteAddr + 2;//<--
		pBuffer++;
		NumToWrite--;
	}while((status == NOR_SUCCESS)  && (NumToWrite != 0));

	return (status);
}

uint8_t NOR_Read_8b ( uint32_t ReadAddr )
{
	NOR_WRITE(ADDR_SHIFT(0x00555), 0x00AA);
	NOR_WRITE(ADDR_SHIFT(0x002AA), 0x0055);
	NOR_WRITE((NOR_FLASH_START_ADDR + ReadAddr), 0x00F0 );

	return (*(__IO uint16_t *)((NOR_FLASH_START_ADDR + ReadAddr)));
}

uint16_t NOR_Read_16b ( uint32_t ReadAddr )
{
	NOR_WRITE(ADDR_SHIFT(0x00555), 0x00AA);
	NOR_WRITE(ADDR_SHIFT(0x002AA), 0x0055);
	NOR_WRITE((NOR_FLASH_START_ADDR + ReadAddr), 0x00F0 );

	return (*(__IO uint16_t *)((NOR_FLASH_START_ADDR + ReadAddr)));
}

uint32_t NOR_Read_32b ( uint32_t ReadAddr )
{
	NOR_WRITE(ADDR_SHIFT(0x00555), 0x00AA);
	NOR_WRITE(ADDR_SHIFT(0x002AA), 0x0055);
	NOR_WRITE((NOR_FLASH_START_ADDR + ReadAddr), 0x00F0 );


	//!< Read a dword from the NOR
	return *(__IO uint32_t *)((NOR_FLASH_START_ADDR + ReadAddr));
}

void NOR_ReadBuffer_8b (uint8_t* pBuffer, uint32_t ReadAddr, uint32_t NumToRead )
{
	NOR_WRITE(ADDR_SHIFT(0x0555), 0x00AA);
	NOR_WRITE(ADDR_SHIFT(0x02AA), 0x0055);
	NOR_WRITE((NOR_FLASH_START_ADDR + ReadAddr), 0x00F0);

	for(; NumToRead != 0x00; NumToRead--) //!< while there is data to read
	{
		//!< Read a Halfword from the NOR
		*pBuffer++ = *(__IO uint16_t *)((NOR_FLASH_START_ADDR + ReadAddr));
		ReadAddr = ReadAddr + 2;
	}
}

void NOR_ReadBuffer_16b ( uint16_t* pBuffer, uint32_t ReadAddr, uint32_t NumToRead )
{
	NOR_WRITE(ADDR_SHIFT(0x0555), 0x00AA);
	NOR_WRITE(ADDR_SHIFT(0x02AA), 0x0055);
	NOR_WRITE((NOR_FLASH_START_ADDR + ReadAddr), 0x00F0);

	for(; NumToRead != 0x00; NumToRead--) //!< while there is data to read
	{
		//!< Read a Halfword from the NOR
		*pBuffer++ = *(__IO uint16_t *)((NOR_FLASH_START_ADDR + ReadAddr));
		ReadAddr = ReadAddr + 2;
	}
}

void NOR_ReadBuffer_32b ( uint32_t* pBuffer, uint32_t ReadAddr, uint32_t NumToRead )
{
	NOR_WRITE(ADDR_SHIFT(0x0555), 0x00AA);
	NOR_WRITE(ADDR_SHIFT(0x02AA), 0x0055);
	NOR_WRITE((NOR_FLASH_START_ADDR + ReadAddr), 0x00F0);

	for(; NumToRead != 0x00; NumToRead--) //!< while there is data to read
	{
		//!< Read a Halfword from the NOR
		*pBuffer++ = *(__IO uint32_t *)((NOR_FLASH_START_ADDR + ReadAddr));
		ReadAddr = ReadAddr + 4;
	}
}

NOR_Status NOR_ReturnToReadMode ( void )
{
	NOR_WRITE(NOR_FLASH_START_ADDR, 0x00F0);

	return (NOR_SUCCESS);
}

NOR_Status NOR_Reset ( void )
{
	NOR_WRITE(ADDR_SHIFT(0x00555), 0x00AA);
	NOR_WRITE(ADDR_SHIFT(0x002AA), 0x0055);
	NOR_WRITE(NOR_FLASH_START_ADDR, 0x00F0);

	return (NOR_SUCCESS);
}

NOR_Status NOR_GetStatus ( uint32_t Timeout )
{
	uint16_t val1 = 0x00, val2 = 0x00;
	NOR_Status status = NOR_ONGOING;
	uint32_t timeout = Timeout;

	//!< Poll on NOR memory Ready/Busy signal ----------------------------------
	while((GPIO_ReadInputDataBit(GPIOD, GPIO_Pin_6) != RESET) && (timeout > 0))
	{
		timeout--;
	}

	timeout = Timeout;

	while((GPIO_ReadInputDataBit(GPIOD, GPIO_Pin_6) == RESET) && (timeout > 0))
	{
		timeout--;
	}

	//!< Get the NOR memory operation status -----------------------------------
	while((Timeout != 0x00) && (status != NOR_SUCCESS))
	{
		Timeout--;

		//!< Read DQ6 and DQ5
		val1 = *(__IO uint16_t *)(NOR_FLASH_START_ADDR);
		val2 = *(__IO uint16_t *)(NOR_FLASH_START_ADDR);

		//!< If DQ6 did not toggle between the two reads then return NOR_Success
		if((val1 & 0x0040) == (val2 & 0x0040))
		{
			return NOR_SUCCESS;
		}

		if((val1 & 0x0020) != 0x0020)
		{
			status = NOR_ONGOING;
		}

		val1 = *(__IO uint16_t *)(NOR_FLASH_START_ADDR);
		val2 = *(__IO uint16_t *)(NOR_FLASH_START_ADDR);

		if((val1 & 0x0040) == (val2 & 0x0040))
		{
			return NOR_SUCCESS;
		}
		else if((val1 & 0x0020) == 0x0020)
		{
			return NOR_ERROR;
		}
	}

	if(Timeout == 0x00)
	{
		status = NOR_TIMEOUT;
	}

	//!< Return the operation status
	return (status);
}*/

/***********************************************************
 * Definitions internal functions
 ***********************************************************/
void SDRAM_NOR_FLASH_GPIOs_init ( void )
{
	GPIO_InitTypeDef GPIO_InitStruct;

	// Activate clocks
	RCC_AHB1PeriphClockCmd( RCC_AHB1Periph_GPIOD, ENABLE );
	RCC_AHB1PeriphClockCmd( RCC_AHB1Periph_GPIOE, ENABLE );
	RCC_AHB1PeriphClockCmd( RCC_AHB1Periph_GPIOF, ENABLE );
	RCC_AHB1PeriphClockCmd( RCC_AHB1Periph_GPIOG, ENABLE );
	RCC_AHB1PeriphClockCmd( RCC_AHB1Periph_GPIOH, ENABLE );
	RCC_AHB1PeriphClockCmd( RCC_AHB1Periph_GPIOI, ENABLE );

	GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStruct.GPIO_Speed = GPIO_Speed_100MHz;

	// PD0  = FMC_D2, PD1  = FMC_D3
	// PD4 = FMC_NOE (NOR_G), PD5 = FMC_NWE (NOR_W), PD7 = FMC_NE1 (NOR_E)
	// PD8  = FMC_D13, PD9  = FMC_D14, PD10 = FMC_D15
	// PD11 = FMC_A16, PD12 = FMC_A17, PD13 = FMC_A18
	// PD14 = FMC_D0,  PD15 = FMC_D1
	GPIO_InitStruct.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_7 | 	//Pin 4, 5 and 7 only for NOR Flash required
			GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 |
			GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
	GPIO_Init( GPIOD, &GPIO_InitStruct );

	// PD6 = FMC_NWAIT (NOR_RB) mit PullUp da Opendrain, danach Pull-Up wieder zuruecksetzen
	GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_UP;
	GPIO_InitStruct.GPIO_Pin = GPIO_Pin_6;
	GPIO_Init( GPIOD, &GPIO_InitStruct );

	GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;

	GPIO_PinAFConfig( GPIOD, GPIO_PinSource0, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOD, GPIO_PinSource1, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOD, GPIO_PinSource4, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOD, GPIO_PinSource5, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOD, GPIO_PinSource6, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOD, GPIO_PinSource7, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOD, GPIO_PinSource8, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOD, GPIO_PinSource9, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOD, GPIO_PinSource10, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOD, GPIO_PinSource11, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOD, GPIO_PinSource12, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOD, GPIO_PinSource13, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOD, GPIO_PinSource14, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOD, GPIO_PinSource15, GPIO_AF_FMC );

	// PE0  = FMC_NBL0, PE1  = FMC_NBL1
	// PE3 = FMC_A19, PE4 = FMC_A20, PE5 = FMC_A21, PE6 = FMC_A22
	// PE7  = FMC_D4, PE8  = FMC_D5, PE9  = FMC_D6, PE10 = FMC_D7,
	// PE11 = FMC_D8, PE12 = FMC_D9, PE13 = FMC_D10, PE14 = FMC_D11,
	// PE15 = FMC_D12
	GPIO_InitStruct.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_3 | GPIO_Pin_4 |
			GPIO_Pin_5 | GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9 |
			GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 |
			GPIO_Pin_15;
	GPIO_Init( GPIOE, &GPIO_InitStruct );

	GPIO_PinAFConfig( GPIOE, GPIO_PinSource0, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOE, GPIO_PinSource1, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOE, GPIO_PinSource3, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOE, GPIO_PinSource4, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOE, GPIO_PinSource5, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOE, GPIO_PinSource6, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOE, GPIO_PinSource7, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOE, GPIO_PinSource8, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOE, GPIO_PinSource9, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOE, GPIO_PinSource10, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOE, GPIO_PinSource11, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOE, GPIO_PinSource12, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOE, GPIO_PinSource13, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOE, GPIO_PinSource14, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOE, GPIO_PinSource15, GPIO_AF_FMC );

	// PF0  = FMC_A0, PF1  = FMC_A1, PF2  = FMC_A2, PF3  = FMC_A3
	// PF4  = FMC_A4, PF5  = FMC_A5
	// PF11 = SDNRAS, PF12 = FMC_A6, PF13 = FMC_A7, PF14 = FMC_A8,
	// PF15 = FMC_A9
	GPIO_InitStruct.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3 |
			GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 |
			GPIO_Pin_14 | GPIO_Pin_15;
	GPIO_Init( GPIOF, &GPIO_InitStruct );

	GPIO_PinAFConfig( GPIOF, GPIO_PinSource0, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOF, GPIO_PinSource1, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOF, GPIO_PinSource2, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOF, GPIO_PinSource3, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOF, GPIO_PinSource4, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOF, GPIO_PinSource5, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOF, GPIO_PinSource11, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOF, GPIO_PinSource12, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOF, GPIO_PinSource13, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOF, GPIO_PinSource14, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOF, GPIO_PinSource15, GPIO_AF_FMC );

	// PG0  = FMC_A10, PG1  = FMC_A11, PG2 = FMC_A12, PG3 = FMC_A13
	// PG4  = FMC_A14 (FMC_BA0), PG5  = FMC_A15 (FMC_BA1)
	// PG8  = SDCLK, PG15 = SDNCAS
	GPIO_InitStruct.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3 |
			GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_8 | GPIO_Pin_15;
	GPIO_Init( GPIOG, &GPIO_InitStruct );

	GPIO_PinAFConfig( GPIOG, GPIO_PinSource0, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOG, GPIO_PinSource1, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOG, GPIO_PinSource2, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOG, GPIO_PinSource3, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOG, GPIO_PinSource4, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOG, GPIO_PinSource5, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOG, GPIO_PinSource8, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOG, GPIO_PinSource15, GPIO_AF_FMC );

	// PH2  = SDNCKE, PH3  = SDNE0, PH5  = SDNWE
	// PH8  = FMC_D16, PH9  = FMC_D17, PH10 = FMC_D18, PH11 = FMC_D19
	// PH12 = FMC_D20, PH13 = FMC_D21, PH14 = FMC_D22, PH15 = FMC_D23
	GPIO_InitStruct.GPIO_Pin = GPIO_Pin_2 | GPIO_Pin_3 | GPIO_Pin_5 | GPIO_Pin_8 |
			GPIO_Pin_9 | GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 |
			GPIO_Pin_14 | GPIO_Pin_15;
	GPIO_Init( GPIOH, &GPIO_InitStruct );

	GPIO_PinAFConfig( GPIOH, GPIO_PinSource2, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOH, GPIO_PinSource3, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOH, GPIO_PinSource5, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOH, GPIO_PinSource8, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOH, GPIO_PinSource9, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOH, GPIO_PinSource10, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOH, GPIO_PinSource11, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOH, GPIO_PinSource12, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOH, GPIO_PinSource13, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOH, GPIO_PinSource14, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOH, GPIO_PinSource15, GPIO_AF_FMC );

	// PI0  = FMC_D24, PI1  = FMC_D25, PI2  = FMC_D26, PI3  = FMC_D27,
	// PI4  = FMC_NBL2, PI5  = FMC_NBL3, PI6  = FMC_D28, PI7  = FMC_D29
	// PI9  = FMC_D30, PI10 = FMC_D31
	GPIO_InitStruct.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3 |
			GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_9 |
			GPIO_Pin_10;
	GPIO_Init( GPIOI, &GPIO_InitStruct );

	GPIO_PinAFConfig( GPIOI, GPIO_PinSource0, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOI, GPIO_PinSource1, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOI, GPIO_PinSource2, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOI, GPIO_PinSource3, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOI, GPIO_PinSource4, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOI, GPIO_PinSource5, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOI, GPIO_PinSource6, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOI, GPIO_PinSource7, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOI, GPIO_PinSource9, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOI, GPIO_PinSource10, GPIO_AF_FMC );
}

void SDRAM_Initialization_Sequence( void )
{
	FMC_SDRAMCommandTypeDef FMC_SDRAMCommand;
	uint32_t tmpmrd = 0;
	uint32_t count = 0;

	// Step1: Configure a clock configuration enable command
	FMC_SDRAMCommand.FMC_CommandMode = FMC_Command_Mode_CLK_Enabled;
	FMC_SDRAMCommand.FMC_CommandTarget = FMC_Command_Target_bank1;
	FMC_SDRAMCommand.FMC_AutoRefreshNumber = 1;
	FMC_SDRAMCommand.FMC_ModeRegisterDefinition = 0;

	while( FMC_GetFlagStatus( FMC_Bank1_SDRAM, FMC_FLAG_Busy ) != RESET );
	FMC_SDRAMCmdConfig( &FMC_SDRAMCommand );

	// Step2: 100us minimum delay
	for ( count = 0; count < 180000; ++count )
		asm("nop");

	// Step3: Configure PALL command
	FMC_SDRAMCommand.FMC_CommandMode = FMC_Command_Mode_PALL;
	FMC_SDRAMCommand.FMC_CommandTarget = FMC_Command_Target_bank1;
	FMC_SDRAMCommand.FMC_AutoRefreshNumber = 1;
	FMC_SDRAMCommand.FMC_ModeRegisterDefinition = 0;

	while( FMC_GetFlagStatus( FMC_Bank1_SDRAM, FMC_FLAG_Busy ) != RESET );
	FMC_SDRAMCmdConfig( &FMC_SDRAMCommand );

	// Step4: Configure Auto Refresh command
	FMC_SDRAMCommand.FMC_CommandMode = FMC_Command_Mode_AutoRefresh;
	FMC_SDRAMCommand.FMC_CommandTarget = FMC_Command_Target_bank1;
	FMC_SDRAMCommand.FMC_AutoRefreshNumber = 8;
	FMC_SDRAMCommand.FMC_ModeRegisterDefinition = 0;

	while( FMC_GetFlagStatus( FMC_Bank1_SDRAM, FMC_FLAG_Busy ) != RESET );
	FMC_SDRAMCmdConfig( &FMC_SDRAMCommand );

	while( FMC_GetFlagStatus( FMC_Bank1_SDRAM, FMC_FLAG_Busy ) != RESET );
	FMC_SDRAMCmdConfig( &FMC_SDRAMCommand );

	// Step5: Programm external memory mode register
	tmpmrd = (uint32_t) SDRAM_MODEREG_BURST_LENGTH_1 |
            			SDRAM_MODEREG_BURST_TYPE_SEQUENTIAL   |
						SDRAM_MODEREG_CAS_LATENCY_3           |
						SDRAM_MODEREG_OPERATING_MODE_STANDARD |
						SDRAM_MODEREG_WRITEBURST_MODE_SINGLE;
	FMC_SDRAMCommand.FMC_CommandMode = FMC_Command_Mode_LoadMode;
	FMC_SDRAMCommand.FMC_CommandTarget = FMC_Command_Target_bank1;
	FMC_SDRAMCommand.FMC_AutoRefreshNumber = 1;
	FMC_SDRAMCommand.FMC_ModeRegisterDefinition = tmpmrd;

	while( FMC_GetFlagStatus( FMC_Bank1_SDRAM, FMC_FLAG_Busy ) != RESET );
	FMC_SDRAMCmdConfig( &FMC_SDRAMCommand );

	FMC_SetRefreshCount( SDRAM_REFRESH_COUNT );

	while( FMC_GetFlagStatus( FMC_Bank1_SDRAM, FMC_FLAG_Busy ) != RESET );
}
/*
void NOR_Initialization_Sequence ( void )
{
	// Todo RAT: Read NOR_ID???
	FMC_NORSRAMCmd(FMC_Bank1_NORSRAM1, ENABLE);
}*/
